@@ -17,14 +17,11 @@
 
 #include <grpc/support/port_platform.h>
 
-#include <stddef.h>
 #include <stdint.h>
 
 #include <algorithm>
 #include <atomic>
-#include <functional>
 #include <memory>
-#include <type_traits>
 #include <utility>
 
 #include "absl/base/thread_annotations.h"
@@ -37,6 +34,7 @@
 
 #include "src/core/lib/gpr/tls.h"
 #include "src/core/lib/gprpp/construct_destruct.h"
+#include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/sync.h"
 #include "src/core/lib/promise/context.h"
 #include "src/core/lib/promise/detail/promise_factory.h"
@@ -63,13 +61,13 @@ class Wakeable {
 class Waker {
  public:
   explicit Waker(Wakeable* wakeable) : wakeable_(wakeable) {}
-  Waker() : wakeable_(&unwakeable_) {}
-  ~Waker() { wakeable_->Drop(); }
+  Waker() : Waker(nullptr) {}
+  ~Waker() {
+    if (wakeable_ != nullptr) wakeable_->Drop();
+  }
   Waker(const Waker&) = delete;
   Waker& operator=(const Waker&) = delete;
-  Waker(Waker&& other) noexcept : wakeable_(other.wakeable_) {
-    other.wakeable_ = &unwakeable_;
-  }
+  Waker(Waker&& other) noexcept : wakeable_(other.Take()) {}
   Waker& operator=(Waker&& other) noexcept {
     std::swap(wakeable_, other.wakeable_);
     return *this;
@@ -77,8 +75,7 @@ class Waker {
 
   // Wake the underlying activity.
   void Wakeup() {
-    wakeable_->Wakeup();
-    wakeable_ = &unwakeable_;
+    if (auto* wakeable = Take()) wakeable->Wakeup();
   }
 
   template <typename H>
@@ -91,14 +88,9 @@ class Waker {
   }
 
  private:
-  class Unwakeable final : public Wakeable {
-   public:
-    void Wakeup() final {}
-    void Drop() final {}
-  };
+  Wakeable* Take() { return absl::exchange(wakeable_, nullptr); }
 
   Wakeable* wakeable_;
-  static Unwakeable unwakeable_;
 };
 
 // An Activity tracks execution of a single promise.
@@ -110,22 +102,8 @@ class Waker {
 // Activity execution may be cancelled by simply deleting the activity. In such
 // a case, if execution had not already finished, the done callback would be
 // called with absl::CancelledError().
-class Activity : private Wakeable {
+class Activity : public Orphanable {
  public:
-  // Cancel execution of the underlying promise.
-  virtual void Cancel() ABSL_LOCKS_EXCLUDED(mu_) = 0;
-
-  // Destroy the Activity - used for the type alias ActivityPtr.
-  struct Deleter {
-    void operator()(Activity* activity) {
-      activity->Cancel();
-      activity->Unref();
-    }
-  };
-
-  // Fetch the size of the implementation of this activity.
-  virtual size_t Size() = 0;
-
   // Force wakeup from the outside.
   // This should be rarely needed, and usages should be accompanied with a note
   // on why it's not possible to wakeup with a Waker object.
@@ -133,11 +111,8 @@ class Activity : private Wakeable {
   // an Activity to repoll.
   void ForceWakeup() { MakeOwningWaker().Wakeup(); }
 
-  // Wakeup the current threads activity - will force a subsequent poll after
-  // the one that's running.
-  static void WakeupCurrent() {
-    current()->SetActionDuringRun(ActionDuringRun::kWakeup);
-  }
+  // Force the current activity to immediately repoll if it doesn't complete.
+  virtual void ForceImmediateRepoll() = 0;
 
   // Return the current activity.
   // Additionally:
@@ -146,122 +121,46 @@ class Activity : private Wakeable {
   //   locked
   // - back up that assertation with a runtime check in debug builds (it's
   //   prohibitively expensive in non-debug builds)
-  static Activity* current() ABSL_ASSERT_EXCLUSIVE_LOCK(current()->mu_) {
-#ifndef NDEBUG
-    GPR_ASSERT(g_current_activity_);
-    if (g_current_activity_ != nullptr) {
-      g_current_activity_->mu_.AssertHeld();
-    }
-#endif
-    return g_current_activity_;
-  }
+  static Activity* current() { return g_current_activity_; }
 
   // Produce an activity-owning Waker. The produced waker will keep the activity
   // alive until it's awoken or dropped.
-  Waker MakeOwningWaker() {
-    Ref();
-    return Waker(this);
-  }
+  virtual Waker MakeOwningWaker() = 0;
 
   // Produce a non-owning Waker. The waker will own a small heap allocated weak
   // pointer to this activity. This is more suitable for wakeups that may not be
   // delivered until long after the activity should be destroyed.
-  Waker MakeNonOwningWaker() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
+  virtual Waker MakeNonOwningWaker() = 0;
 
  protected:
-  // Action received during a run, in priority order.
-  // If more than one action is received during a run, we use max() to resolve
-  // which one to report (so Cancel overrides Wakeup).
-  enum class ActionDuringRun : uint8_t {
-    kNone,    // No action occured during run.
-    kWakeup,  // A wakeup occured during run.
-    kCancel,  // Cancel was called during run.
-  };
-
-  inline virtual ~Activity() {
-    if (handle_) {
-      DropHandle();
-    }
-  }
-
-  // All promise execution occurs under this mutex.
-  Mutex mu_;
-
   // Check if this activity is the current activity executing on the current
   // thread.
   bool is_current() const { return this == g_current_activity_; }
   // Check if there is an activity executing on the current thread.
   static bool have_current() { return g_current_activity_ != nullptr; }
-  // Check if we got an internal wakeup since the last time this function was
-  // called.
-  ActionDuringRun GotActionDuringRun() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
-    return absl::exchange(action_during_run_, ActionDuringRun::kNone);
-  }
-
   // Set the current activity at construction, clean it up at destruction.
   class ScopedActivity {
    public:
-    explicit ScopedActivity(Activity* activity) {
-      GPR_ASSERT(g_current_activity_ == nullptr);
+    explicit ScopedActivity(Activity* activity)
+        : prior_activity_(g_current_activity_) {
       g_current_activity_ = activity;
     }
-    ~ScopedActivity() { g_current_activity_ = nullptr; }
+    ~ScopedActivity() { g_current_activity_ = prior_activity_; }
     ScopedActivity(const ScopedActivity&) = delete;
     ScopedActivity& operator=(const ScopedActivity&) = delete;
+
+   private:
+    Activity* const prior_activity_;
   };
 
-  // Implementors of Wakeable::Wakeup should call this after the wakeup has
-  // completed.
-  void WakeupComplete() { Unref(); }
-
-  // Mark the current activity as being cancelled (so we can actually cancel it
-  // after polling).
-  void CancelCurrent() {
-    current()->SetActionDuringRun(ActionDuringRun::kCancel);
-  }
-
  private:
-  class Handle;
-
-  void Ref() { refs_.fetch_add(1, std::memory_order_relaxed); }
-  void Unref() {
-    if (1 == refs_.fetch_sub(1, std::memory_order_acq_rel)) {
-      delete this;
-    }
-  }
-
-  // Return a Handle instance with a ref so that it can be stored waiting for
-  // some wakeup.
-  Handle* RefHandle() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
-  // If our refcount is non-zero, ref and return true.
-  // Otherwise, return false.
-  bool RefIfNonzero();
-  // Drop the (proved existing) wait handle.
-  void DropHandle() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
-  // Set the action that occured during this run.
-  // We use max to combine actions so that cancellation overrides wakeups.
-  void SetActionDuringRun(ActionDuringRun action)
-      ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
-    action_during_run_ = std::max(action_during_run_, action);
-  }
-
-  // Current refcount.
-  std::atomic<uint32_t> refs_{1};
-  // If wakeup is called during Promise polling, we set this to Wakeup and
-  // repoll. If cancel is called during Promise polling, we set this to Cancel
-  // and cancel at the end of polling.
-  ActionDuringRun action_during_run_ ABSL_GUARDED_BY(mu_) =
-      ActionDuringRun::kNone;
-  // Handle for long waits. Allows a very small weak pointer type object to
-  // queue for wakeups while Activity may be deleted earlier.
-  Handle* handle_ ABSL_GUARDED_BY(mu_) = nullptr;
   // Set during RunLoop to the Activity that's executing.
   // Being set implies that mu_ is held.
   static GPR_THREAD_LOCAL(Activity*) g_current_activity_;
 };
 
 // Owned pointer to one Activity.
-using ActivityPtr = std::unique_ptr<Activity, Activity::Deleter>;
+using ActivityPtr = OrphanablePtr<Activity>;
 
 namespace promise_detail {
 
@@ -320,6 +219,107 @@ class ActivityContexts : public ContextHolder<Contexts>... {
   };
 };
 
+// A free standing activity: an activity that owns its own synchronization and
+// memory.
+// The alternative is an activity that's somehow tied into another system, for
+// instance the type seen in promise_based_filter.h as we're transitioning from
+// the old filter stack to the new system.
+// FreestandingActivity is-a Wakeable, but needs to increment a refcount before
+// returning that Wakeable interface. Additionally, we want to keep
+// FreestandingActivity as small as is possible, since it will be used
+// everywhere. So we use inheritance to provide the Wakeable interface: this
+// makes it zero sized, and we make the inheritance private to prevent
+// accidental casting.
+class FreestandingActivity : public Activity, private Wakeable {
+ public:
+  Waker MakeOwningWaker() final {
+    Ref();
+    return Waker(this);
+  }
+  Waker MakeNonOwningWaker() final;
+
+  void Orphan() final {
+    Cancel();
+    Unref();
+  }
+
+  void ForceImmediateRepoll() final {
+    mu_.AssertHeld();
+    SetActionDuringRun(ActionDuringRun::kWakeup);
+  }
+
+ protected:
+  // Action received during a run, in priority order.
+  // If more than one action is received during a run, we use max() to resolve
+  // which one to report (so Cancel overrides Wakeup).
+  enum class ActionDuringRun : uint8_t {
+    kNone,    // No action occured during run.
+    kWakeup,  // A wakeup occured during run.
+    kCancel,  // Cancel was called during run.
+  };
+
+  inline ~FreestandingActivity() override {
+    if (handle_) {
+      DropHandle();
+    }
+  }
+
+  // Check if we got an internal wakeup since the last time this function was
+  // called.
+  ActionDuringRun GotActionDuringRun() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+    return absl::exchange(action_during_run_, ActionDuringRun::kNone);
+  }
+
+  // Implementors of Wakeable::Wakeup should call this after the wakeup has
+  // completed.
+  void WakeupComplete() { Unref(); }
+
+  // Set the action that occured during this run.
+  // We use max to combine actions so that cancellation overrides wakeups.
+  void SetActionDuringRun(ActionDuringRun action)
+      ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+    action_during_run_ = std::max(action_during_run_, action);
+  }
+
+  Mutex* mu() ABSL_LOCK_RETURNED(mu_) { return &mu_; }
+
+ private:
+  class Handle;
+
+  // Cancel execution of the underlying promise.
+  virtual void Cancel() = 0;
+
+  void Ref() { refs_.fetch_add(1, std::memory_order_relaxed); }
+  void Unref() {
+    if (1 == refs_.fetch_sub(1, std::memory_order_acq_rel)) {
+      delete this;
+    }
+  }
+
+  // Return a Handle instance with a ref so that it can be stored waiting for
+  // some wakeup.
+  Handle* RefHandle() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
+  // If our refcount is non-zero, ref and return true.
+  // Otherwise, return false.
+  bool RefIfNonzero();
+  // Drop the (proved existing) wait handle.
+  void DropHandle() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
+
+  // All promise execution occurs under this mutex.
+  Mutex mu_;
+
+  // Current refcount.
+  std::atomic<uint32_t> refs_{1};
+  // If wakeup is called during Promise polling, we set this to Wakeup and
+  // repoll. If cancel is called during Promise polling, we set this to Cancel
+  // and cancel at the end of polling.
+  ActionDuringRun action_during_run_ ABSL_GUARDED_BY(mu_) =
+      ActionDuringRun::kNone;
+  // Handle for long waits. Allows a very small weak pointer type object to
+  // queue for wakeups while Activity may be deleted earlier.
+  Handle* handle_ ABSL_GUARDED_BY(mu_) = nullptr;
+};
+
 // Implementation details for an Activity of an arbitrary type of promise.
 // There should exist a static function:
 // struct WakeupScheduler {
@@ -331,14 +331,20 @@ class ActivityContexts : public ContextHolder<Contexts>... {
 // It can assume that activity will remain live until RunScheduledWakeup() is
 // invoked, and that a given activity will not be concurrently scheduled again
 // until its RunScheduledWakeup() has been invoked.
+// We use private inheritance here as a way of getting private members for
+// each of the contexts.
+// TODO(ctiller): We can probably reconsider the private inheritance here
+// when we move away from C++11 and have more powerful template features.
 template <class F, class WakeupScheduler, class OnDone, typename... Contexts>
-class PromiseActivity final : public Activity,
+class PromiseActivity final : public FreestandingActivity,
                               private ActivityContexts<Contexts...> {
  public:
   using Factory = PromiseFactory<void, F>;
+  using ResultType = typename Factory::Promise::Result;
+
   PromiseActivity(F promise_factory, WakeupScheduler wakeup_scheduler,
                   OnDone on_done, Contexts&&... contexts)
-      : Activity(),
+      : FreestandingActivity(),
         ActivityContexts<Contexts...>(std::forward<Contexts>(contexts)...),
         wakeup_scheduler_(std::move(wakeup_scheduler)),
         on_done_(std::move(on_done)) {
@@ -346,9 +352,9 @@ class PromiseActivity final : public Activity,
     // This may hit a waiter, which could expose our this pointer to other
     // threads, meaning we do need to hold this mutex even though we're still
     // constructing.
-    mu_.Lock();
+    mu()->Lock();
     auto status = Start(Factory(std::move(promise_factory)));
-    mu_.Unlock();
+    mu()->Unlock();
     // We may complete immediately.
     if (status.has_value()) {
       on_done_(std::move(*status));
@@ -362,16 +368,24 @@ class PromiseActivity final : public Activity,
     GPR_ASSERT(done_);
   }
 
-  size_t Size() override { return sizeof(*this); }
+  void RunScheduledWakeup() {
+    GPR_ASSERT(wakeup_scheduled_.exchange(false, std::memory_order_acq_rel));
+    Step();
+    WakeupComplete();
+  }
+
+ private:
+  using typename ActivityContexts<Contexts...>::ScopedContext;
 
   void Cancel() final {
     if (Activity::is_current()) {
-      CancelCurrent();
+      mu()->AssertHeld();
+      SetActionDuringRun(ActionDuringRun::kCancel);
       return;
     }
     bool was_done;
     {
-      MutexLock lock(&mu_);
+      MutexLock lock(mu());
       // Check if we were done, and flag done.
       was_done = done_;
       if (!done_) MarkDone();
@@ -382,15 +396,6 @@ class PromiseActivity final : public Activity,
     }
   }
 
-  void RunScheduledWakeup() {
-    GPR_ASSERT(wakeup_scheduled_.exchange(false, std::memory_order_acq_rel));
-    Step();
-    WakeupComplete();
-  }
-
- private:
-  using typename ActivityContexts<Contexts...>::ScopedContext;
-
   // Wakeup this activity. Arrange to poll the activity again at a convenient
   // time: this could be inline if it's deemed safe, or it could be by passing
   // the activity to an external threadpool to run. If the activity is already
@@ -400,7 +405,8 @@ class PromiseActivity final : public Activity,
     // If there is an active activity, but hey it's us, flag that and we'll loop
     // in RunLoop (that's calling from above here!).
     if (Activity::is_current()) {
-      WakeupCurrent();
+      mu()->AssertHeld();
+      SetActionDuringRun(ActionDuringRun::kWakeup);
       WakeupComplete();
       return;
     }
@@ -418,7 +424,7 @@ class PromiseActivity final : public Activity,
 
   // Notification that we're no longer executing - it's ok to destruct the
   // promise.
-  void MarkDone() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+  void MarkDone() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu()) {
     GPR_ASSERT(!done_);
     done_ = true;
     Destruct(&promise_holder_.promise);
@@ -426,16 +432,16 @@ class PromiseActivity final : public Activity,
 
   // In response to Wakeup, run the Promise state machine again until it
   // settles. Then check for completion, and if we have completed, call on_done.
-  void Step() ABSL_LOCKS_EXCLUDED(mu_) {
+  void Step() ABSL_LOCKS_EXCLUDED(mu()) {
     // Poll the promise until things settle out under a lock.
-    mu_.Lock();
+    mu()->Lock();
     if (done_) {
       // We might get some spurious wakeups after finishing.
-      mu_.Unlock();
+      mu()->Unlock();
       return;
     }
     auto status = RunStep();
-    mu_.Unlock();
+    mu()->Unlock();
     if (status.has_value()) {
       on_done_(std::move(*status));
     }
@@ -444,7 +450,7 @@ class PromiseActivity final : public Activity,
   // The main body of a step: set the current activity, and any contexts, and
   // then run the main polling loop. Contained in a function by itself in
   // order to keep the scoping rules a little easier in Step().
-  absl::optional<absl::Status> RunStep() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+  absl::optional<ResultType> RunStep() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu()) {
     ScopedActivity scoped_activity(this);
     ScopedContext contexts(this);
     return StepLoop();
@@ -453,8 +459,8 @@ class PromiseActivity final : public Activity,
   // Similarly to RunStep, but additionally construct the promise from a
   // promise factory before entering the main loop. Called once from the
   // constructor.
-  absl::optional<absl::Status> Start(Factory promise_factory)
-      ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+  absl::optional<ResultType> Start(Factory promise_factory)
+      ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu()) {
     ScopedActivity scoped_activity(this);
     ScopedContext contexts(this);
     Construct(&promise_holder_.promise, promise_factory.Once());
@@ -463,7 +469,7 @@ class PromiseActivity final : public Activity,
 
   // Until there are no wakeups from within and the promise is incomplete:
   // poll the promise.
-  absl::optional<absl::Status> StepLoop() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_) {
+  absl::optional<ResultType> StepLoop() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu()) {
     GPR_ASSERT(is_current());
     while (true) {
       // Run the promise.
@@ -493,7 +499,7 @@ class PromiseActivity final : public Activity,
   // Callback on completion of the promise.
   GPR_NO_UNIQUE_ADDRESS OnDone on_done_;
   // Has execution completed?
-  GPR_NO_UNIQUE_ADDRESS bool done_ ABSL_GUARDED_BY(mu_) = false;
+  GPR_NO_UNIQUE_ADDRESS bool done_ ABSL_GUARDED_BY(mu()) = false;
   // Is there a wakeup scheduled?
   GPR_NO_UNIQUE_ADDRESS std::atomic<bool> wakeup_scheduled_{false};
   // We wrap the promise in a union to allow control over the construction
@@ -504,7 +510,7 @@ class PromiseActivity final : public Activity,
     ~PromiseHolder() {}
     GPR_NO_UNIQUE_ADDRESS Promise promise;
   };
-  GPR_NO_UNIQUE_ADDRESS PromiseHolder promise_holder_ ABSL_GUARDED_BY(mu_);
+  GPR_NO_UNIQUE_ADDRESS PromiseHolder promise_holder_ ABSL_GUARDED_BY(mu());
 };
 
 }  // namespace promise_detail
